package com.zh.project0512.aop;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.zh.project0512.annotation.AdminTokenValid;
import com.zh.project0512.annotation.TokenValid;
import com.zh.project0512.utils.JwtUtil;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.jsonwebtoken.Claims;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.annotation.Resource;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

@Component
public class LoginTokenInterceptor implements HandlerInterceptor {

    private static final String TOKEN = "token";
    ObjectMapper objectMapper = new ObjectMapper();
    @Resource
    private JwtUtil jwtUtil;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {

        // 判断访问的方法上是否有@needToken注解
        boolean tokenValid =false;
        try {
            HandlerMethod handlerMethod = (HandlerMethod) handler;
            tokenValid = handlerMethod.hasMethodAnnotation(AdminTokenValid.class);
        }catch (Exception e){
            response.setStatus(500);
            this.respUrlError(response);
            return false;
        }
        if (!tokenValid) {
            return true;
        }
        String token = request.getHeader(TOKEN);
        // 若token为空,且需要token
        if (StringUtils.isEmpty(token)) {
            response.setStatus(500);
            this.respFail(response);
            return false;
        }
        Claims claims = jwtUtil.parseJWT(token);
        if (claims == null) {
            response.setStatus(502);
            this.respFail(response);
            return false;
        }
        return true;
    }

    /**
     * 返回登录失败结果Json数据
     */
    private void respFail(ServletResponse response) throws IOException {
        Result err = Result.fail(HttpStatusEnum.LOGIN_FAILED);
        String s = objectMapper.writeValueAsString(err);
        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/json; charset=utf-8");
        response.getOutputStream().write(s.getBytes());
    }
    /**
     * 返回handler解析错误（路由地址错误）
     */
    private void respUrlError(ServletResponse response) throws IOException {
        Result err = Result.fail(HttpStatusEnum.NOT_FOUND);
        String s = objectMapper.writeValueAsString(err);
        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/json; charset=utf-8");
        response.getOutputStream().write(s.getBytes());
    }
}
