package com.zh.project0512.controller.manage;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.zh.project0512.annotation.AdminCheckAuthorityAnnotation;
import com.zh.project0512.annotation.AdminTokenValid;
import com.zh.project0512.model.Admin;
import com.zh.project0512.model.dto.AdminJurisdictionFBIDTO;
import com.zh.project0512.model.vo.AdminJurisdictionFBRIVo;
import com.zh.project0512.service.IAdminJurisdictionService;
import com.zh.project0512.service.IAdminService;
import com.zh.project0512.utils.*;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-05-18
 */
@Tag(name = "后台账号管理")
@RestController
@RequestMapping("manage/admin")
public class AdminController {

    @Autowired
    private IAdminService adminService;
    @Resource
    IAdminJurisdictionService userJurisdictionService;
    @Data
    static class LoginParam {
        @NotNull(message = "name不能为空")
        @Schema(title = "账号名称")
        private String name;
        @NotNull(message = "密码不能为空")
        @Schema(title = "密码")
        private String password;
    }

    @Operation(summary = "登录")
    @PostMapping("/login")
    public Result login(@Validated @RequestBody LoginParam param) {
        String name = param.getName();
        String password = param.getPassword();
        QueryWrapper<Admin> qw = new QueryWrapper<>();
        qw.eq("name", name);
        Admin admin = adminService.getOne(qw);
        if (null == admin) {
            return Result.fail(HttpStatusEnum.USER_NOT_EXIST, "未注册！");
        }
        if (!password.equals(admin.getPassword())) {
            return Result.fail(HttpStatusEnum.USER_NOT_EXIST, "密码错误！");
        }
        String token = new JwtUtil().createAdminJWT(admin.getId().toString());
        admin.setToken(token).setLastLoginAt(LocalDateTime.now());
        UpdateWrapper<Admin> uw = new UpdateWrapper<>();
        uw.eq("id", admin.getId()).set("token", token).set("lastLoginAt", LocalDateTime.now());
        adminService.update(uw);

        JSONObject ad = (JSONObject) JSONObject.toJSON(admin);
        String roleIdsStr = admin.getRoleIds();
        if (roleIdsStr != null && !roleIdsStr.trim().equals("")) {
            String[] roleIds = roleIdsStr.split(",");
            if (roleIds.length > 1) {
                return Result.success(ad,"暂不支持多角色的用户菜单列表展示");
            }
        }
        AdminJurisdictionFBIDTO adminJurisdictionFBIDTO = new AdminJurisdictionFBIDTO();
        adminJurisdictionFBIDTO.setRoleId(Integer.valueOf(roleIdsStr));
        List<AdminJurisdictionFBRIVo> byRoleId = userJurisdictionService.findByRoleId(adminJurisdictionFBIDTO);
        ad.put("role",byRoleId);
        return Result.success(ad);
    }

    @Data
    static class registerParam {
        @NotNull(message = "name不能为空")
        @Schema(title = "账号名称")
        private String name;
        @NotNull(message = "密码不能为空")
        @Schema(title = "密码")
        @NotNull(message = "密码不能为空")
        private String tel;
        @Schema(title = "密码")
        private String password;
        @Schema(title = "角色id", description = "X多个角色用','分隔X")
        private String roleIds;
        @Schema(title = "真实名称")
        private String realName;
    }

    @Operation(summary = "添加")
    @PostMapping("register")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "125")
    public Result register(@Validated @RequestBody registerParam param) {
        QueryWrapper<Admin> qw = new QueryWrapper<>();
        qw.eq("name", param.getName());
        if (null != adminService.getOne(qw)) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "已注册！");
        }
        Admin admin = new Admin().setName(param.getName()).setPassword(param.getPassword()).setTel(param.getTel())
            .setRealName(param.getRealName()).setRoleIds(param.getRoleIds()).setCreatAt(LocalDateTime.now());
        adminService.save(admin);
        return Result.success(admin);
    }


    @Data
    static class editParam {
        @NotNull(message = "账号id不能为空")
        @Schema(title = "账号id")
        private String id;
        @Schema(title = "账号名称")
        private String name;
        @Schema(title = "密码")
        private String tel;
        @Schema(title = "密码")
        private String password;
        @Schema(title = "角色id", description = "多个角色用','分隔")
        private String roleIds;
        @Schema(title = "真实名称")
        private String realName;
    }

    @Operation(summary = "编辑")
    @PostMapping("edit")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "126")
    public Result edit(@Validated @RequestBody editParam param) {
        UpdateWrapper<Admin> up = new UpdateWrapper<>();
        up.eq("id", param.getId());
        if (null == adminService.getOne(up)) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "未找到该用户！");
        }
        up.set(param.getName() != null, "name", param.getName());
        up.set(param.getTel() != null, "tel", param.getTel());
        up.set(param.getPassword() != null, "password", param.getPassword());
        up.set(param.getRoleIds() != null, "roleIds", param.getRoleIds());
        up.set(param.getRealName() != null, "realName", param.getRealName());
        up.set("updateAt", LocalDateTime.now());
        return Result.success(adminService.update(up));
    }

    @Data
    static class AdminStatusParam {
        @NotNull(message = "账号id不能为空")
        @Schema(title = "账号id")
        private String id;
//        @NotNull(message = "open不能为空")
//        @Schema(title = "开启或禁用",description = "open为true为开启；为false为禁用")
//        private boolean open;
    }

    @Operation(summary = "切换账户状态")
    @PostMapping("switchStatus")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "127")
    public Result switchStatus(@Validated @RequestBody AdminStatusParam param) {
//        int status = param.open?1:2;
        UpdateWrapper<Admin> up = new UpdateWrapper<>();
        up.eq("id", param.getId());
        Admin admin = adminService.getOne(up);
        if (null == admin) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "未找到该用户！");
        }
//        if(admin.getStatus() == status) {
//            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "未发生改动！");
//        }
        up.set("status", admin.getStatus() == 1 ? 2 : 1);
        up.set("updateAt", LocalDateTime.now());
        return Result.success(adminService.update(up), "修改完成");
    }

    @Data
    static class adminListParam {
        @Schema(title = "账号名称")
        private String name;
        @Schema(title = "手机号")
        private String tel;
        @Schema(title = "真实名称")
        private String realName;
        private Integer pageNum;
        private Integer pageSize;
    }

    @Operation(summary = "列表")
    @PostMapping("list")
    @AdminTokenValid
    public Result list(@Validated @RequestBody adminListParam param) {
        QueryWrapper<Admin> qw = new QueryWrapper<>();
        qw.like(param.getName() != null, "name", param.getName());
        qw.like(param.getTel() != null, "tel", param.getTel());
        qw.like(param.getRealName() != null, "realName", param.getRealName());
        return Result.success(adminService.page(MybatisPlusUtil.SetNumPage(param.getPageNum(), param.getPageSize()), qw));
    }


    @Data
    static class adminDetParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "账号id")
        private Integer id;
    }

    @Operation(summary = "账号详情")
    @PostMapping("detail")
    @AdminTokenValid
    public Result detail(@Validated @RequestBody adminDetParam param) {
        Admin admin = adminService.getById(param.getId());
        return Result.success(admin);
    }
}
