package com.zh.project0512.controller.manage;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.zh.project0512.annotation.AdminCheckAuthorityAnnotation;
import com.zh.project0512.annotation.AdminTokenValid;
import com.zh.project0512.model.*;
import com.zh.project0512.model.vo.FileUploadParamVo;
import com.zh.project0512.service.*;
import com.zh.project0512.utils.FileTypeUtil;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.io.File;
import java.sql.Ref;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-05-23
 */
@RestController
@RequestMapping("/manage/reference")
@Tag(name = "素材")
public class ReferenceController {
    @Autowired
    private IReferenceService referenceService;
    @Autowired
    private ITaskReferenceService taskReferenceService;
    @Autowired
    private IReferenceTagService referenceTagService;
    @Autowired
    private IReferenceBrandService referenceBrandService;
    @Autowired
    private ITopicActivityService topicActivityService;
    @Value("${web.uploadPath}")
    private String path;

    @Data
    static class AddRParam {
        @NotNull(message = "标题不能为空")
        @Schema(title = "标题")
        private String title;
        @Schema(title = "素材分组id", description = "0为未分组")
        private Integer groupId;
        @Schema(title = "文件地址列表")
        @NotEmpty(message = "fileUrl列表不能为空")
        private List<FileUploadParamVo> fileUrlList;
        @Schema(title = "封面图片地址", description = "视频文件请上传封面图片")
        private String coverUrl;
        @Schema(title = "时长", description = "视频文件请上传时长")
        private Integer duration;
        @Schema(title = "标签id列表")
        private List<Integer> tagList;
        @Schema(title = "品牌id列表")
        private List<Integer> brandList;
    }

    @Operation(summary = "添加素材", description = "可根据文件不同同时上传多个素材")
    @PostMapping("/add")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "107")
    public Result add(@Validated @RequestBody AddRParam param) {
        List<FileUploadParamVo> list = param.getFileUrlList();
        List<Reference> l = new ArrayList<>();
//        List<TopicActivity> ta = new ArrayList<>();
        List<Integer> tagList = param.getTagList();
        List<Integer> brandList = param.getBrandList();
        for (FileUploadParamVo e : list) {
            String url = e.getUrl();
            if (url != null && url.lastIndexOf(".") != -1) {
                l.add(new Reference()
                    .setTitle(param.getTitle())
                    .setGroupId(param.getGroupId())
                    .setCoverUrl(param.getCoverUrl())
                    .setCreatAt(LocalDateTime.now())
                    .setDuration(e.getDuration())
                    .setFileUrl(url)
                    .setFileUrlSmall(e.getFileUrlSmall())
                    .setType(FileTypeUtil.getcontentTypeNum(url.substring(url.lastIndexOf("."))))
                );
//                ta.add(new TopicActivity().setTitle(param.getTitle()).setCover(param.getCoverUrl()).setCreatAt(LocalDateTime.now())
//                    .setContent(e).setType(FileTypeUtil.getcontentTypeNum(e.substring(e.lastIndexOf(".")))));
            }
        }
        if (l.size() < 1) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "无有效数据");
        }
        referenceService.saveBatch(l);
//        topicActivityService.saveBatch(ta);
        if (tagList != null && tagList.size() > 0) {
            List<JSONObject> tag = new ArrayList<>();
            for (Reference r : l) {
                for (Integer t : tagList) {
                    tag.add(new JSONObject().fluentPut("referenceId", r.getId()).fluentPut("tagId", t));
                }
            }
            referenceTagService.addGroup(tag);
        }
        List<JSONObject> brand = new ArrayList<>();
        if (brandList != null && brandList.size() > 0) {
            for (Reference r : l) {
                for (Integer t : brandList) {
                    brand.add(new JSONObject().fluentPut("referenceId", r.getId()).fluentPut("brandId", t));
                }
            }
            referenceBrandService.addGroup(brand);
        }
        return Result.success("添加完成！");
    }

    @Data
    static class AddTagParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "素材id")
        private Integer id;
        @NotNull(message = "tagId不能为空")
        @Min(value = 1, message = "tagId最小值为1")
        @Schema(title = "标签id")
        private Integer tagId;
    }

    @Operation(summary = "素材新增标签")
    @PostMapping("/addTag")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "108")
    public Result addTag(@Validated @RequestBody AddTagParam param) {
        if (null != referenceTagService.getMap(Wrappers.<ReferenceTag>query().lambda().eq(ReferenceTag::getReferenceId, param.getId()).eq(ReferenceTag::getTagId, param.getTagId()))) {
            return Result.success("数据已存在");
        }
        referenceTagService.saveOrUpdate(new ReferenceTag().setReferenceId(param.getId()).setTagId(param.getTagId()));
        return Result.success("添加完成");
    }

    @Data
    static class AddBrandParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "素材id")
        private Integer id;
        @NotNull(message = "brandId不能为空")
        @Min(value = 1, message = "brandId最小值为1")
        @Schema(title = "品牌id")
        private Integer brandId;
    }

    @Operation(summary = "素材新增品牌")
    @PostMapping("/addBrand")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "108")
    public Result addBrand(@Validated @RequestBody AddBrandParam param) {
        if (null != referenceBrandService.getMap(Wrappers.<ReferenceBrand>query().lambda().eq(ReferenceBrand::getReferenceId, param.getId()).eq(ReferenceBrand::getBrandId, param.getBrandId()))) {
            return Result.success("数据已存在");
        }
        referenceBrandService.saveOrUpdate(new ReferenceBrand().setReferenceId(param.getId()).setBrandId(param.getBrandId()));
        return Result.success("添加完成");
    }

    @Data
    static class DelRParam {
        @NotNull(message = "id数组不能为空")
        @Schema(title = "素材Id数组")
        private List<Integer> idList;
    }

    @Operation(summary = "删除素材(软删除),文件会清除")
    @PostMapping("/del")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "109")
    public Result del(@Validated @RequestBody DelRParam param) {
        List<Reference> rList = referenceService.list(new QueryWrapper<Reference>().in("id", param.getIdList()));
        // 删除素材字段
        referenceService.update(new UpdateWrapper<Reference>().in("id", param.getIdList()).set("isDeleted", 1));
        //删除素材文件
        for (Reference r : rList) {
            deleteFile(r.getFileUrl());
            deleteFile(r.getCoverUrl());
        }
//        //删除素材任务关联
//        taskReferenceService.remove(new UpdateWrapper<TaskReference>().in("referenceId", param.getIdList()));
        return MybatisPlusUtil.sqlResult(referenceService.update(new UpdateWrapper<Reference>().in("id", param.getIdList()).set("isDeleted", 1)), "删除");

    }

    @Data
    static class DelTagParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "素材id")
        private Integer id;
        @NotNull(message = "tagId不能为空")
        @Min(value = 1, message = "tagId最小值为1")
        @Schema(title = "标签id")
        private Integer tagId;
    }

    @Operation(summary = "素材删除标签")
    @PostMapping("/delTag")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "108")
    public Result delTag(@Validated @RequestBody DelTagParam param) {
        return MybatisPlusUtil.sqlResult(referenceTagService.remove(Wrappers.<ReferenceTag>query().lambda().eq(ReferenceTag::getReferenceId, param.getId()).eq(ReferenceTag::getTagId, param.getTagId())), "删除");
    }

    @Data
    static class DelBrandParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "素材id")
        private Integer id;
        @NotNull(message = "brandId不能为空")
        @Min(value = 1, message = "brandId最小值为1")
        @Schema(title = "品牌id")
        private Integer brandId;
    }

    @Operation(summary = "素材删除品牌")
    @PostMapping("/delBrand")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "108")
    public Result delBrand(@Validated @RequestBody DelBrandParam param) {
        return MybatisPlusUtil.sqlResult(referenceBrandService.remove(Wrappers.<ReferenceBrand>query().lambda().eq(ReferenceBrand::getReferenceId, param.getId()).eq(ReferenceBrand::getBrandId, param.getBrandId())), "删除");
    }

    @Data
    static class UpdRParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "素材id")
        private Integer id;
        @Schema(title = "名称")
        private String title;
        @Schema(title = "文件地址")
        private String fileUrl;
        @Schema(title = "文件缩略图（图片超高清时方便预加载）")
        private String fileUrlSmall;
        @Schema(title = "时长(视频需要)")
        private Integer duration;
        @Schema(title = "封面图片地址")
        private String coverUrl;
        @Schema(title = "素材分组id", description = "0为未分组")
        private Integer groupId;
        @Schema(title = "标签id列表")
        private List<Integer> tagList;
        @Schema(title = "品牌id列表")
        private List<Integer> brandList;
    }

    @Operation(summary = "更新素材")
    @PostMapping("/upd")
    @Transactional
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "108")
    public Result upd(@Validated @RequestBody UpdRParam param) {
        int id = param.getId();
        // 添加任务标签
        List tagIdList = param.getTagList();
        if (tagIdList != null) {
            if (tagIdList.size() == 0) {
                referenceTagService.remove(new QueryWrapper<ReferenceTag>().eq("referenceId", id));
            } else {
                List<ReferenceTag> tagListOri = referenceTagService.list(new QueryWrapper<ReferenceTag>().eq("referenceId", id));
                referenceTagService.remove(new QueryWrapper<ReferenceTag>().eq("referenceId", id).notIn((tagIdList != null && tagIdList.size() != 0), "tagId", tagIdList));
                List tagIdListOri = new ArrayList();
                for (ReferenceTag rt : tagListOri) {
                    tagIdListOri.add(rt.getTagId());
                }
                List<Integer> collect = (List) tagIdList.stream().filter(item -> !tagIdListOri.contains(item)).collect(Collectors.toList());
                if (collect != null && collect.size() != 0) {
                    List<JSONObject> tag = new ArrayList<>();
                    for (Integer t : collect) {
                        tag.add(new JSONObject().fluentPut("referenceId", id).fluentPut("tagId", t));
                    }
                    referenceTagService.addGroup(tag);
                }
            }
        }

        // 添加任务品牌
        List brandIdList = param.getBrandList();
        if (brandIdList != null) {
            if (brandIdList.size() == 0) {
                referenceBrandService.remove(new QueryWrapper<ReferenceBrand>().eq("referenceId", id));
            } else {
                List<ReferenceBrand> brandListOri = referenceBrandService.list(new QueryWrapper<ReferenceBrand>().eq("referenceId", id));
                referenceBrandService.remove(new QueryWrapper<ReferenceBrand>().eq("referenceId", id).notIn("brandId", brandIdList));
                List brandIdListOri = new ArrayList();
                for (ReferenceBrand rb : brandListOri) {
                    brandIdListOri.add(rb.getBrandId());
                }
                List<Integer> collect = (List) brandIdList.stream().filter(item -> !brandIdListOri.contains(item)).collect(Collectors.toList());
                if (collect != null && collect.size() != 0) {
                    List<JSONObject> brand = new ArrayList<>();
                    for (Integer t : collect) {
                        brand.add(new JSONObject().fluentPut("referenceId", id).fluentPut("brandId", t));
                    }
                    referenceBrandService.addGroup(brand);

                }
            }
        }
        Reference reference = new Reference()
            .setTitle(param.getTitle())
            .setFileUrl(param.getFileUrl())
            .setFileUrlSmall(param.getFileUrlSmall())
            .setCoverUrl(param.getCoverUrl())
            .setGroupId(param.getGroupId())
            .setUpdateAt(LocalDateTime.now());
        if (param.getFileUrl() != null) {
            String fileName = param.getFileUrl();
            reference.setType(FileTypeUtil.getcontentTypeNum(fileName.substring(fileName.lastIndexOf("."))));
        }
        UpdateWrapper<Reference> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", id);
        return MybatisPlusUtil.sqlResult(referenceService.update(reference, updateWrapper), "修改");
    }

    @Data
    static class ListRParam {
        private int pageNum;
        private int pageSize;
        @Schema(title = "素材分组id")
        private Integer groupId;
        @Schema(title = "月份区间")
        private Integer month;
        @Schema(title = "开始时间(yyyy-MM-dd HH:mm:ss)", description = "如果传入month以月份区间为条件")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime beginTime;
        @Schema(title = "结束时间(yyyy-MM-dd HH:mm:ss)", description = "如果传入month以月份区间为条件")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime endTime;
        @Schema(title = "tagId数组")
        private List<Integer> tagIdList;
        @Schema(title = "brandId数组")
        private List<Integer> brandIdList;
        @Schema(title = "任务名称关键词搜索")
        private String keyword;
    }

    @Operation(summary = "素材列表")
    @PostMapping("/list")
    @AdminTokenValid
    public Result list(@Validated @RequestBody ListRParam param) {
        QueryWrapper<Reference> qw = new QueryWrapper<>();
        if (null != param.getGroupId()) {
            qw.eq("t1.groupId", param.getGroupId());
        }
        if (null != param.getMonth()) {
            qw.between("t1.creatAt", LocalDateTime.now().minusMonths(param.getMonth()), LocalDateTime.now());
        } else {
            qw.ge(param.getBeginTime() != null, "t1.creatAt", param.getBeginTime());
            qw.le(param.getEndTime() != null, "t1.creatAt", param.getEndTime());
        }
        if (null != param.getKeyword()) {
            qw.like("t1.title", param.getKeyword());
        }
        IPage iPage = referenceService.idList(MybatisPlusUtil.SetNumPage(param.getPageNum(), param.getPageSize()), param.getTagIdList(), param.getBrandIdList(), qw);
        List list = iPage.getRecords();
        if (list.size() > 0) {
            iPage.setRecords(referenceService.list(null, list, param.getTagIdList(), param.getBrandIdList(), qw));
        }
        return Result.success(iPage);
    }

    @Data
    static class DetPParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "素材id")
        private Integer id;
    }

    @Operation(summary = "详情")
    @PostMapping("/detail")
    @AdminTokenValid
    public Result detail(@Validated @RequestBody DetPParam param) {
        int id = param.getId();
        Reference reference = referenceService.getById(id);
        return reference != null ? Result.success(reference.setBrandList(referenceBrandService.selByReferenceId(id)).setTagList(referenceTagService.selByReferenceId(id))) : Result.fail(HttpStatusEnum.NOT_FOUND);
    }

    void deleteFile(String fileUrl) {
        File file = new File(path + "/upload" + fileUrl);
        if (fileUrl != null) {
            System.out.println("fileDelete================"+file);
            file.delete();
        }
    }
}
