package com.zh.project0512.controller.manage;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.zh.project0512.annotation.AdminTokenValid;
import com.zh.project0512.mapper.AdminMapper;
import com.zh.project0512.mapper.UserEffectRecordsMapper;
import com.zh.project0512.model.UserEffectRecords;
import com.zh.project0512.model.Video;
import com.zh.project0512.model.dto.RankListDTO;
import com.zh.project0512.service.IRankService;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.time.temporal.TemporalAdjusters;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Stream;

/**
 * 首页数据统计
 */
@Tag(name = "首页数据统计")
@RestController
@RequestMapping("/manage/statistics")
public class StatisticsController {
    @Autowired
    private AdminMapper adminMapper;
    @Autowired
    private UserEffectRecordsMapper userEffectRecordsMapper;

    @Operation(summary = "总数据")
    @PostMapping("/data1")
    @AdminTokenValid
    public Result data1() {
        return Result.success(adminMapper.statistics1(), "请求成功");
    }

    @Data
    static class SSParam {
        @Schema(title = "时间条件", description = "1今日;2本周;3本月;4全年")
        private int dateCondition;
        @Schema(title = "开始时间(yyyy-MM-dd HH:mm:ss)", description = "如果传入dateCondition以月份区间为条件")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime beginTime;
        @Schema(title = "结束时间(yyyy-MM-dd HH:mm:ss)", description = "如果传入dateCondition以月份区间为条件")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime endTime;
    }

    @Operation(summary = "视频数据")
    @PostMapping("/data2")
    @AdminTokenValid
    public Result data2(@Validated @RequestBody SSParam param) {
        Integer condition = param.getDateCondition();
        System.out.println(condition);
        QueryWrapper<Video> qw = new QueryWrapper<>();
        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        DateTimeFormatter df2 = DateTimeFormatter.ofPattern("yyyy-MM");
        LocalDate now = LocalDateTime.now().toLocalDate();
        LocalDate before =now.with(TemporalAdjusters.firstDayOfMonth());
        if (null != condition) {
            switch (condition) {
                case 1:
                    before = now;
                    break;
                case 2:
                    before = now.with(DayOfWeek.MONDAY);
                    break;
                case 3:
                    before = now.with(TemporalAdjusters.firstDayOfMonth());
                    break;
                case 4:
                    before = now.with(TemporalAdjusters.firstDayOfYear());
                    break;
                default:
            }
        } else {
            before = param.getBeginTime() != null ? param.getBeginTime().toLocalDate() : before;
            now = param.getEndTime() != null ? param.getEndTime().toLocalDate() : now;
        }
        ArrayList<JSONObject> res = new ArrayList<>();
        if(condition == 4){
            qw.between("date_format(creatAt, '%Y-%m')", before.format(df2), now.plusMonths(1).format(df2));
            List<JSONObject> videoDataBydDay =adminMapper.statistics3(qw);
            if (videoDataBydDay == null || videoDataBydDay.size() == 0) {
                return Result.success(null, "请求成功");
            }
            for (int i = 0; i <=   ChronoUnit.MONTHS.between(before,now); i++) {
                LocalDate today = before.plusMonths(i);
                JSONObject obj = null;
//            Stream<JSONObject> dat1 = videoDataBydDay.stream().filter(x -> LocalDate.parse((CharSequence) x.get("dat"), df) == today);
                for (JSONObject o : videoDataBydDay) {
                    if (o.get("dat").equals(today.format(df2))) {
                        obj = o;
                        break;
                    }
                }
                res.add(obj == null ? new JSONObject().fluentPut("dat", today.format(df2)).fluentPut("playNumSum",0).fluentPut("collectionNumSum",0).fluentPut("reSendNumSum",0)
                    .fluentPut("recommendNumSum",0).fluentPut("commentNumSum",0).fluentPut("commendNumSum",0): obj);
            }
        }else {
            qw.between("date_format(creatAt, '%Y-%m-%d')", before, now.plusDays(1));
            List<JSONObject> videoDataBydDay = adminMapper.statistics2(qw);
            if (videoDataBydDay == null || videoDataBydDay.size() == 0) {
                return Result.success(null, "请求成功");
            }
            for (int i = 0; i <= ChronoUnit.DAYS.between(before, now); i++) {
                LocalDate today = before.plusDays(i);
                JSONObject obj = null;
//            Stream<JSONObject> dat1 = videoDataBydDay.stream().filter(x -> LocalDate.parse((CharSequence) x.get("dat"), df) == today);
                for (JSONObject o : videoDataBydDay) {
                    if (LocalDate.parse((CharSequence) o.get("dat"), df).equals(today)) {
                        obj = o;
                        break;
                    }
                }
                res.add(obj == null ? new JSONObject().fluentPut("dat", today).fluentPut("playNumSum",0).fluentPut("collectionNumSum",0).fluentPut("reSendNumSum",0)
                    .fluentPut("recommendNumSum",0).fluentPut("commentNumSum",0).fluentPut("commendNumSum",0) : obj);
            }
        }
        return Result.success(res, "请求成功");
    }

    @Data
    static class RankParam {
        @Schema(title = "时间条件", description = "1今日;2本周;3本月;4全年")
        private Integer dateCondition;
        @Schema(title = "1经销商2员工", description = "默认1")
        private Integer type;
        private Integer pageNum;
        private Integer pageSize;
    }

    @Operation(summary = "排行列表")
    @PostMapping("/data3")
    @AdminTokenValid
    public Result data3(@Validated @RequestBody RankParam param) {

        Page pg = MybatisPlusUtil.SetNumPage(param.getPageNum(), param.getPageSize());
        QueryWrapper<UserEffectRecords> qw = new QueryWrapper<>();
        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        LocalDateTime now = LocalDateTime.of(LocalDateTime.now().toLocalDate(), LocalTime.MIN);
        LocalDateTime before = now.with(TemporalAdjusters.firstDayOfMonth());
        if (param.getDateCondition() != null) {
            switch (param.getDateCondition()) {
                case 1:
                    before = now;
                    break;
                case 2:
                    before = now.with(DayOfWeek.MONDAY);
                    break;
                case 3:
                    before = now.with(TemporalAdjusters.firstDayOfMonth());
                    break;
                case 4:
                    before = now.with(TemporalAdjusters.firstDayOfYear());
                    break;
                default:
            }
        }
        qw.between("createAt", before, now.plusDays(1));
        IPage page =(param.getType() != null && param.getType() == 1) ? userEffectRecordsMapper.rankByDepartment(pg,qw) : userEffectRecordsMapper.rankByUser(pg,qw);
        return Result.success(page, "请求成功"); }
}
