package com.zh.project0512.controller.manage;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.zh.project0512.annotation.AdminCheckAuthorityAnnotation;
import com.zh.project0512.annotation.AdminTokenValid;
import com.zh.project0512.mapper.UserMapper;
import com.zh.project0512.model.*;
import com.zh.project0512.model.validatedDemo.TaskValidGroup1;
import com.zh.project0512.model.validatedDemo.TaskValidGroup2;
import com.zh.project0512.model.vo.FileUploadParamVo;
import com.zh.project0512.service.*;
import com.zh.project0512.utils.*;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneOffset;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-05-21
 */
@Tag(name = "任务管理")
@Validated
@RestController
@RequestMapping("/manage/task")
public class TaskController {
    @Autowired
    private IUserService userService;
    @Autowired
    private ITaskService taskService;
    @Autowired
    private IUserTaskService userTaskService;
    @Autowired
    private ITaskTagService taskTagService;
    @Autowired
    private ITaskBrandService taskBrandService;
    @Autowired
    private ITaskReferenceService taskReferenceService;
    @Autowired
    private IRewardRuleService rewardRuleService;
    @Autowired
    private IRewardRuleTemplateService rewardRuleTemplateService;
    @Autowired
    private ICustomerActionNoteService customerActionNoteService;
    @Autowired
    private IReferenceService referenceService;
    @Autowired
    private ITopicActivityService topicActivityService;
    @Autowired
    private ITeachingRefeService teachingRefeService;
    @Autowired
    private ITaskTeachRefeService taskTeachRefeService;
    @Autowired
    private ITaskDepartmentService taskDepartmentService;
    @Autowired
    private IVideoService videoService;
    @Autowired
    private IQywxDepartmentGroupLinkService qywxDepartmentGroupLinkService;

    @Operation(summary = "新增")
    @PostMapping("/add")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "102")
    @Transactional
    public Result add(@Validated(TaskValidGroup1.class) @RequestBody Task task) {
        if (LocalDateTime.now().isAfter(task.getStart())) {
            task.setStatus(1);
        }
        task.setStart( LocalDateTime.of(task.getStart().toLocalDate(), LocalTime.MIN));
        task.setEnd( (LocalDateTime.of(task.getEnd().toLocalDate(), LocalTime.MIN).plusDays(1).minusSeconds(1)));
        task.setCreatAt(LocalDateTime.now()).setUpdateAt(LocalDateTime.now());
        taskService.save(task);
        int id = task.getId();
        List tagList = task.getTagList();
        if (tagList != null && tagList.size() > 0) {
            taskTagService.addGroup(tagList, id);
        }
        List brandList = task.getBrandList();
        if (brandList != null && brandList.size() > 0) {
            taskBrandService.addGroup(brandList, id);
        }
        List referenceList = task.getReferenceList();
        if (referenceList != null && referenceList.size() > 0) {
            taskReferenceService.addGroup(referenceList, id);
        }
        List departmentList = task.getDepartmentList();
        if (departmentList != null && departmentList.size() > 0) {
            taskDepartmentService.addGroup(departmentList, id);
        }
        List departmentGroupList = task.getDepartmentGroupList();
        if (departmentGroupList != null && departmentGroupList.size() > 0) {
            List departmentIdList = qywxDepartmentGroupLinkService.departmentIdList(null, departmentGroupList);
            taskDepartmentService.addGroup(departmentIdList, id);
        }
        // 上传任务时同时上传素材
        List<FileUploadParamVo> referenceUrlList = task.getReferenceUrlList();
        if (referenceUrlList != null && referenceUrlList.size() > 0) {
            List<Reference> l = new ArrayList<>();
//            List<TopicActivity> ta = new ArrayList<>();
            for (FileUploadParamVo e : referenceUrlList) {
                String url = e.getUrl();
                if (url != null && e.getUrl().lastIndexOf(".") != -1) {
                    l.add(new Reference().setTitle(task.getTitle()).setCoverUrl(task.getCoverUrl()).setDuration(e.getDuration()).setCreatAt(LocalDateTime.now()).setUpdateAt(LocalDateTime.now())
                        .setFileUrl(url).setType(FileTypeUtil.getcontentTypeNum(url.substring(url.lastIndexOf(".")))).setIsDeleted(1));
//                    ta.add(new TopicActivity().setTitle(task.getTitle()).setCover(task.getCoverUrl()).setCreatAt(LocalDateTime.now())
//                        .setContent(e).setType(FileTypeUtil.getcontentTypeNum(e.substring(e.lastIndexOf(".")))));
                }
            }
            if (l.size() > 0) {
                referenceService.saveBatch(l);
//                topicActivityService.saveBatch(ta);
                List<Integer> reference = new ArrayList<>();
                for (Reference r : l) {
                    reference.add(r.getId());
                }
                taskReferenceService.addGroup(reference, id);
            }
        }
        // 上传任务的通用教程
        List<TaskTeachRefe> l2 = new ArrayList<>();
        List<Integer> teachRefeList = task.getTeachRefeList();
        if (teachRefeList != null && teachRefeList.size() > 0) {
            List<TeachingRefe> teachingRefes = (List<TeachingRefe>) teachingRefeService.listByIds(teachRefeList);
            for (TeachingRefe e : teachingRefes) {
                l2.add((new TaskTeachRefe().setTaskId(id).setCreatAt(LocalDateTime.now())
                    .setFileUrl(e.getFileUrl()).setType(FileTypeUtil.getcontentTypeNum(e.getFileUrl().substring(e.getFileUrl().lastIndexOf("."))))));
            }
        }
        List<FileUploadParamVo> teachRefeUrlList = task.getTeachRefeUrlList();
        if (teachRefeUrlList != null && teachRefeUrlList.size() > 0) {
            for (FileUploadParamVo e : teachRefeUrlList) {
                String url = e.getUrl();
                if (url != null && url.lastIndexOf(".") != -1) {
                    l2.add(new TaskTeachRefe().setTaskId(id).setCreatAt(LocalDateTime.now()).setDuration(e.getDuration())
                        .setFileUrl(url).setType(FileTypeUtil.getcontentTypeNum(url.substring(url.lastIndexOf(".")))));
                }
            }
        }
        if (l2.size() > 0) {
            taskTeachRefeService.saveBatch(l2);
        }
        return Result.success(null,"添加完成");
    }

    @Data
    static class DelTaskParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "任务id")
        private Integer id;
    }

    @Data
    static class AddTaskTagParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "任务id")
        private Integer id;
        @NotNull(message = "tagId不能为空")
        @Min(value = 1, message = "tagId最小值为1")
        @Schema(title = "标签id")
        private Integer tagId;
    }

    @Operation(summary = "新增任务标签")
    @PostMapping("/addTag")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "144")
    public Result addTag(@Validated @RequestBody AddTaskTagParam param) {
        Integer taskId = param.getId();
        Integer tagId = param.getTagId();
        if (null != taskTagService.getMap(Wrappers.<TaskTag>query().lambda().eq(TaskTag::getTaskId, taskId).eq(TaskTag::getTagId, tagId))) {
            return Result.success("数据已存在");
        }
        taskTagService.saveOrUpdate(new TaskTag().setTaskId(taskId).setTagId(tagId));
        return Result.success("添加完成");
    }

    @Data
    static class AddTaskBrandParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "任务id")
        private Integer id;
        @NotNull(message = "brandId不能为空")
        @Min(value = 1, message = "brandId最小值为1")
        @Schema(title = "品牌id")
        private Integer brandId;
    }

    @Operation(summary = "新增任务品牌")
    @PostMapping("/addBrand")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "144")
    public Result addBrand(@Validated @RequestBody AddTaskBrandParam param) {
        Integer taskId = param.getId();
        Integer brandId = param.getBrandId();
        if (null != taskBrandService.getMap(Wrappers.<TaskBrand>query().lambda().eq(TaskBrand::getTaskId, taskId).eq(TaskBrand::getBrandId, brandId))) {
            return Result.success("数据已存在");
        }
        taskBrandService.saveOrUpdate(new TaskBrand().setTaskId(taskId).setBrandId(brandId).setCreatAt(LocalDateTime.now()));
        return Result.success("添加完成");
    }

    @Data
    static class DelTaskTagParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "任务id")
        private Integer id;
        @NotNull(message = "tagId不能为空")
        @Min(value = 1, message = "tagId最小值为1")
        @Schema(title = "标签id")
        private Integer tagId;
    }

    @Operation(summary = "删除任务标签")
    @PostMapping("/delTag")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "144")
    public Result delTag(@Validated @RequestBody DelTaskTagParam param) {
        return MybatisPlusUtil.sqlResult(taskTagService.remove(Wrappers.<TaskTag>query().lambda().eq(TaskTag::getTaskId, param.getId()).eq(TaskTag::getTagId, param.getTagId())), "删除");
    }

    @Data
    static class DelTaskBrandParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "任务id")
        private Integer id;
        @NotNull(message = "brandId不能为空")
        @Min(value = 1, message = "brandId最小值为1")
        @Schema(title = "品牌id")
        private Integer brandId;
    }

    @Operation(summary = "删除任务品牌")
    @PostMapping("/delBrand")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "144")
    public Result delBrand(@Validated @RequestBody DelTaskBrandParam param) {
        return MybatisPlusUtil.sqlResult(taskBrandService.remove(Wrappers.<TaskBrand>query().lambda().eq(TaskBrand::getTaskId, param.getId()).eq(TaskBrand::getBrandId, param.getBrandId())), "删除");
    }

    @Data
    static class AddTaskReferenceParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "任务id")
        private Integer id;
        @NotNull(message = "referenceId不能为空")
        @Min(value = 1, message = "referenceId最小值为1")
        @Schema(title = "素材Id")
        private Integer referenceId;
    }

    @Operation(summary = "新增任务素材")
    @PostMapping("/addReference")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "144")
    public Result addBrand(@Validated @RequestBody AddTaskReferenceParam param) {
        Integer taskId = param.getId();
        Integer referenceId = param.getReferenceId();
        if (null != taskReferenceService.getMap(Wrappers.<TaskReference>query().lambda().eq(TaskReference::getTaskId, taskId).eq(TaskReference::getReferenceId, referenceId))) {
            return Result.success("数据已存在");
        }
        taskReferenceService.saveOrUpdate(new TaskReference().setTaskId(taskId).setReferenceId(referenceId).setCreatAt(LocalDateTime.now()));
        return Result.success("添加完成");
    }

    @Data
    static class DelTaskReferenceParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "任务id")
        private Integer id;
        @NotNull(message = "referenceId不能为空")
        @Min(value = 1, message = "referenceId最小值为1")
        @Schema(title = "素材id")
        private Integer referenceId;
    }

    @Operation(summary = "删除任务素材")
    @PostMapping("/delReference")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "144")
    public Result delTag(@Validated @RequestBody DelTaskReferenceParam param) {
        return MybatisPlusUtil.sqlResult(taskReferenceService.remove(Wrappers.<TaskReference>query().lambda().eq(TaskReference::getTaskId, param.getId()).eq(TaskReference::getReferenceId, param.getReferenceId())), "删除");
    }

    @Operation(summary = "更新")
    @PostMapping("/upd")
    @AdminTokenValid
    public Result upd(@Validated(TaskValidGroup2.class) @RequestBody Task task) {
        if (task.getStart() != null && LocalDateTime.now().isAfter(task.getStart())) {
            task.setStatus(1);
        }
        int id = task.getId();

        // 添加任务标签
        List tagIdList = task.getTagList();
        if (tagIdList != null) {
            if (tagIdList.size() == 0) {
                taskTagService.remove(new QueryWrapper<TaskTag>().eq("taskId", id));
            } else {
                List<Map> tagListOri = taskTagService.selByTaskId(id);
                taskTagService.remove(new QueryWrapper<TaskTag>().eq("taskId", id).notIn((tagIdList != null && tagIdList.size() != 0), "tagId", tagIdList));
                List tagIdListOri = new ArrayList();
                for (Map map : tagListOri) {
                    tagIdListOri.add(map.get("id"));
                }
                List collect = (List) tagIdList.stream().filter(item -> !tagIdListOri.contains(item)).collect(Collectors.toList());
                if (collect != null && collect.size() != 0) {
                    taskTagService.addGroup(collect, id);
                }
            }
        }

        // 添加任务品牌
        List brandIdList = task.getBrandList();
        if (brandIdList != null) {
            if (brandIdList.size() == 0) {
                taskBrandService.remove(new QueryWrapper<TaskBrand>().eq("taskId", id));
            } else {
                List<Map> brandListOri = taskBrandService.selByTaskId(id);
                taskBrandService.remove(new QueryWrapper<TaskBrand>().eq("taskId", id).notIn("brandId", brandIdList));
                List brandIdListOri = new ArrayList();
                for (Map map : brandListOri) {
                    brandIdListOri.add(map.get("id"));
                }
                List collect = (List) brandIdList.stream().filter(item -> !brandIdListOri.contains(item)).collect(Collectors.toList());
                if (collect != null && collect.size() != 0) {
                    taskBrandService.addGroup(collect, id);
                }
            }
        }

        // 添加任务部门
        List<Integer> departmentIdList = task.getDepartmentList();
        List departmentGroupList = task.getDepartmentGroupList();
        if (departmentIdList != null || departmentGroupList != null) {
            List dpIdList = new ArrayList();
            if (departmentIdList != null && departmentIdList.size() > 0) {
                for (Integer e : departmentIdList) {
                    dpIdList.add(e);
                }
            }
            if (departmentGroupList != null && departmentGroupList.size() > 0) {
                List<Integer> departmentIdList1 = qywxDepartmentGroupLinkService.departmentIdList(null, departmentGroupList);
                if (departmentIdList != null && departmentIdList.size() > 0) {
                    for (Integer e : departmentIdList1) {
                        dpIdList.add(e);
                    }
                }
            }
            if (dpIdList.size() == 0) {
                taskDepartmentService.remove(new QueryWrapper<TaskDepartment>().eq("taskId", id));
            } else {
                List<TaskDepartment> departmentListOri = taskDepartmentService.list(new QueryWrapper<TaskDepartment>().eq("taskId", task.getId()));
                taskDepartmentService.remove(new QueryWrapper<TaskDepartment>().eq("taskId", id).notIn("departmentId", departmentIdList));
                List departmentIdListOri = new ArrayList();
                for (TaskDepartment taskDepartment : departmentListOri) {
                    departmentIdListOri.add(taskDepartment.getDepartmentId());
                }
                List collect = (List) dpIdList.stream().filter(item -> !departmentIdListOri.contains(item)).collect(Collectors.toList());
                if (collect != null && collect.size() != 0) {
                    taskDepartmentService.addGroup(collect, id);
                }
            }
        }
        taskService.upReferenceInTask(task, task.getReferenceList(), task.getReferenceUrlList(), true);
        taskService.upTeachRefeInTask(task, task.getTeachRefeList(), task.getTeachRefeUrlList(), true);
        task.setUpdateAt(LocalDateTime.now());
        taskService.updateById(task);
        return Result.success(null, "修改完成");
    }

    @Operation(summary = "删除")
    @PostMapping("/dele")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "135")
    public Result dele(@Validated @RequestBody DelTaskParam param) {
        Task task = taskService.getById(param.getId());
        if (task.getStatus() != 0) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "只支持未上线的任务删除");
        }
        return MybatisPlusUtil.sqlResult(taskService.removeById(param.getId()), "删除");
    }

    @Operation(summary = "终止任务")
    @PostMapping("/del")
    @AdminTokenValid
    @Transactional
    @AdminCheckAuthorityAnnotation(jurisdictionId = "103")
    public Result del(@Validated @RequestBody DetTaskParam param) {
        UpdateWrapper<Task> ew = new UpdateWrapper<>();
        ew.eq("status", 1).eq("id", param.getId())
            .set("status", 2).set("end", LocalDateTime.now()).set("updateAt", LocalDateTime.now());
        taskService.update(ew);
        videoService.updStatusByTaskId(param.getId(), LocalDateTime.now());
        return Result.success("", "终止任务完成");
    }

    @Autowired
    private UserMapper userMapper;
    @Autowired
    private IUserPointsRecordsService userPointsRecordsService;
    @Autowired
    private HttpUtil httpUtil;

    @PostMapping("/finish")
    @ResponseBody
    public Result finish(@Validated @RequestBody DelTaskParam param) {
        // 任务结束7天时，自动结算积分
        QueryWrapper<Task> qw = new QueryWrapper<>();
        LocalDateTime now = LocalDateTime.now();
        LocalDateTime weekAgo = now.minusWeeks(0);
        AppMessage appMessage = new AppMessage().setTitle("任务积分结算");
        String format = weekAgo.format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        qw.eq("status", 2).eq("id", param.getId());
        List<Task> taskList = taskService.list(qw);
        String qywxAccessToken = httpUtil.qywxGetToken();
        // 处理每个任务的传播值
        for (Task t : taskList) {
            Integer taskId = t.getId();
            List<Map> videoEffectResult = videoService.videoEffectResult(taskId);
            List<Map> rewardRule = rewardRuleService.listByTemplateId(t.getRewardRuleTemplateId());
            List<UserPointsRecords> recordsList = new ArrayList<>();
            // 处理任务中每个用户的传播值
            for (Map m : videoEffectResult) {
                BigDecimal bigDecimal = (BigDecimal) m.get("effectResultSum");
                Integer effectResultSum = Integer.parseInt(bigDecimal.toString());
                if (effectResultSum != null && effectResultSum != 0) {
                    int point = 0;
                    int userId = (Integer) m.get("userId");
                    String touser = (String) m.get("touser");
                    // 根据传播值计算规则算出奖励
                    for (Map r : rewardRule) {
                        if (effectResultSum > (Integer) r.get("limitNum")) {
                            point = (Integer) r.get("reward");
                        }
                    }
                    // （视频结算标识）并更新用户积分,发送企业消息和小程序内消息
                    userMapper.updPoints(userId, point);
                    appMessage.setDescription("恭喜您获得星途积分" + point + "。")
                        .setUrl("pages/quest/subpage/detail/detail?id=" + taskId)
                        .setReceiverId(userId);
                    // 企业微信消息通知
                    if (touser != null) {
                        appMessage.setTouser(touser);
                        List<JSONObject> content_item = new ArrayList<>();
                        content_item.add(new JSONObject().fluentPut("key", "到账结果").fluentPut("value", appMessage.getDescription()));
                        content_item.add(new JSONObject().fluentPut("key", "到账时间").fluentPut("value", LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"))));
                        httpUtil.qywxMessage(qywxAccessToken, touser, "任务积分结算", null, appMessage.getUrl(), content_item);
                    }
                    AppMessageUtil.sendMessage(appMessage);
                    recordsList.add(new UserPointsRecords().setSettlementMethod(1).setPoints(point).setType(1).setRemarks("任务完成结算：" + t.getTitle()).setCreateDate(now).setUserId(userId));
                }
            }
            // 按任务生成积分流水记录
            userPointsRecordsService.saveBatch(recordsList);
            taskService.update(new UpdateWrapper<Task>().eq("id", taskId).set("status", 3));
        }
        return Result.success();
    }

    @Data
    static class ListTParam {
        private int pageNum;
        private int pageSize;
        @Schema(title = "任务状态", description = "0未上线；1已上线；2已结束;3已结算")
        private Integer status;
        @Schema(title = "月份区间")
        private Integer month;
        @Schema(title = "开始时间起(yyyy-MM-dd HH:mm:ss)", description = "如果传入month以月份区间为条件")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime beginTime1;
        @Schema(title = "开始时间末(yyyy-MM-dd HH:mm:ss)", description = "如果传入month以月份区间为条件")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime beginTime2;
        @Schema(title = "结束时间起(yyyy-MM-dd HH:mm:ss)", description = "如果传入month以月份区间为条件")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime endTime1;
        @Schema(title = "结束时间末(yyyy-MM-dd HH:mm:ss)", description = "如果传入month以月份区间为条件")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime endTime2;
        @Schema(title = "tagId数组")
        private List<Integer> tagIdList;
        @Schema(title = "brandId数组")
        private List<Integer> brandIdList;
        @Schema(title = "任务名称关键词搜索")
        private String keyword;
    }

    @Operation(summary = "列表")
    @PostMapping("/list")
    @AdminTokenValid
    public Result list(@Validated @RequestBody ListTParam param) {
        QueryWrapper<Reference> qw = new QueryWrapper<>();
        if (null != param.getStatus()) {
            qw.eq("t1.status", param.getStatus());
        }
        if (null != param.getMonth()) {
            qw.between("t1.start", LocalDateTime.now().minusMonths(param.getMonth()), LocalDateTime.now());
        } else {
            if(null !=param.getBeginTime1()){
                qw.ge("date_format(t1.start, '%Y-%m-%d')", param.getBeginTime1().toLocalDate());
            }
            if(null !=param.getBeginTime2()){
                qw.le( "date_format(t1.start, '%Y-%m-%d')", param.getBeginTime2().toLocalDate());
            }
            if(null !=param.getEndTime1()){
                qw.ge( "date_format(t1.end, '%Y-%m-%d')", param.getEndTime1().toLocalDate());
            }
            if(null !=param.getEndTime2()){
                qw.le( "date_format(t1.end, '%Y-%m-%d')", param.getEndTime2().toLocalDate());
            }
        }
        if (null != param.getKeyword()) {
            qw.like("t1.title", param.getKeyword());
        }
        IPage iPage = taskService.idList(MybatisPlusUtil.SetNumPage(param.getPageNum(), param.getPageSize()), null, param.getTagIdList(), param.getBrandIdList(), qw);
        List list = iPage.getRecords();
        if (list.size() > 0) {
            iPage.setRecords(taskService.list(null, list, param.getTagIdList(), param.getBrandIdList(), qw));
        }
        return Result.success(iPage);
    }

    @Data
    static class DetTaskParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "任务id")
        private Integer id;
    }

    @Operation(summary = "详情")
    @PostMapping("detail")
    @AdminTokenValid
    public Result det(@Validated @RequestBody DetTaskParam param) {
        Task task = taskService.detailById(null, param.getId());
        JSONObject res = (JSONObject) JSONObject.toJSON(task);
        if (task != null) {
            List rewardRulesList = rewardRuleService.listByTemplateId(task.getRewardRuleTemplateId());
            res.put("rewardRuleList", rewardRulesList);
            List departmentList = taskService.taskDepartment(task.getId());
            res.put("departmentList", departmentList);
        }
        return Result.success(res);
    }

    @Operation(summary = "任务传播数据")
    @PostMapping("taskEffect")
    @AdminTokenValid
    public Result taskEffect(@Validated @RequestBody DetTaskParam  param) {
        Integer taskId = param.getId();
        List<Map> maps = taskService.taskEffectData(new QueryWrapper<Task>().eq("t2.id", taskId));
        Map map = maps.get(0);
        if (map != null) {
        map.remove("userId");
        map.remove("userName");
        Integer taskUserSum = 0;
        if (taskDepartmentService.count(new QueryWrapper<TaskDepartment>().eq("taskId", taskId)) == 0) {
            taskUserSum = userService.count();
        }else{
            taskUserSum = taskService.taskUserSum(taskId);
        }
        map.put("taskUserSum",taskUserSum);
        map.put("taskUserNum", taskUserSum =userTaskService.count(new QueryWrapper<UserTask>().eq("taskId",taskId)));
        }
        return Result.success(map);
    }

    @Data
    static class TaskDataParam {
        private Integer pageNum;
        private Integer pageSize;
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "任务id")
        private Integer id;
        @Schema(title = "用户名称")
        private String userName;
        @Schema(title = "部门名称")
        private String departmentName;
        @Schema(title = "开始时间(yyyy-MM-dd HH:mm:ss)")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime beginTime;
        @Schema(title = "结束时间(yyyy-MM-dd HH:mm:ss)")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime endTime;
    }

    @Operation(summary = "任务传播数据")
    @PostMapping("taskEffectData")
    @AdminTokenValid
    public Result taskEffectData(@Validated @RequestBody TaskDataParam param) {
        QueryWrapper<Task> qw = new QueryWrapper<>();
        qw.eq("t2.id", param.getId()).groupBy("t1.userId");
        qw.like(param.getUserName() != null, "t3.name", param.getUserName());
//        qw.like(param.getDepartmentName() != null, "t4.name", param.getDepartmentName());
        qw.inSql(param.getDepartmentName() != null,"t3.openid",
            "SELECT t1.openid from qywxDepartmentUserLink as t1 INNER JOIN qywxDepartment as t2 on t1.departmentId = t2.departmentId and t2.name like "+"'%"+param.getDepartmentName()+"%'");
        if(null !=param.getBeginTime()){
            qw.ge( "date_format(t1.creatAt, '%Y-%m-%d')", param.getBeginTime());
        }
        if(null !=param.getEndTime()){
            qw.le( "date_format(t1.creatAt, '%Y-%m-%d')", param.getEndTime());
        }
        IPage iPage = taskService.taskEffectDataUserIdPage(MybatisPlusUtil.SetNumPage(param.getPageNum(), param.getPageSize()),qw);
        List list = iPage.getRecords();
        qw.orderByDesc("effectResultSum");
        qw.groupBy("t5.departmentId");
        if (list.size() > 0) {
            qw.in("t1.userId",list);
            iPage.setRecords(taskService.taskEffectDataPage( qw));
        }
        return Result.success(iPage);
    }

    @Data
    static class TaskUserParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "任务id")
        private Integer id;
        @NotNull(message = "用户id不能为空")
        @Schema(title = "用户id")
        private Integer userId;
    }

    @Operation(summary = "任务内员工传播数据")
    @PostMapping("taskUserEffect")
    @AdminTokenValid
    public Result taskUserEffect(@Validated @RequestBody TaskUserParam param) {
        QueryWrapper<Task> qw = new QueryWrapper<>();
        qw.eq("t2.id", param.getId()).eq("t1.userId", param.getUserId()).groupBy("t1.userId");
        JSONObject taskEffect = taskService.taskEffect(qw);
        if (taskEffect != null) {
            QueryWrapper<Video> qw2 = new QueryWrapper<>();
            qw2.eq("taskId", param.getId()).eq("userId", param.getUserId());
            taskEffect.put("userEffectDetail", videoService.list(qw2));
        }
        return Result.success(taskEffect);
    }

    @Operation(summary = "导出excel", description = "任务成员传播效果")
    @PostMapping("/excel")
    @AdminTokenValid
    public void excel(HttpServletResponse response, @Validated @RequestBody DetTaskParam param) throws IOException {
        //这是表头及格式
        String[][] array = {
            {"任务名称", "员工姓名", "部门", "播放数", "点赞数", "收藏数", "评论数", "转发数", "推荐数", "传播值"},
            {"taskTitle", "userName", "departmentName", "playNumSum", "commendNumSum", "collectionNumSum", "commentNumSum", "reSendNumSum", "recommendNumSum", "effectResultSum"},
            {"String", "String", "String", "int", "int", "int", "int", "int", "int", "int"}
        };
        QueryWrapper<Task> qw = new QueryWrapper<>();
        qw.eq("t2.id", param.getId()).groupBy("t1.userId");
        List<Map> l = taskService.taskEffectData(qw);

        Workbook wb = ExcelUtil.writeToExcelByList(array, l);
        OutputStream output = response.getOutputStream();
        String fileName = "任务传播数据.xlsx";
        try {
            fileName = URLEncoder.encode(fileName, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        response.setHeader("Content-disposition", "attachment;filename=" + fileName + ";" + "filename*=utf-8''" + fileName);
        wb.write(output);
        output.close();
    }

    @Data
    static class CNListParam {
        @Schema(title = "姓名")
        private String name;
        @Schema(title = "手机号")
        private String tel;
        @Schema(title = "车型")
        private String brand;
        @Schema(title = "提交人")
        private String providerName;
        @Schema(title = "部门id")
        private Integer departmentId;
        private Integer pageNum;
        private Integer pageSize;
    }

    @Operation(summary = "线索搜集列表")
    @PostMapping("/customerNoteList")
    @AdminTokenValid
    public Result customerNote(@RequestBody CNListParam param) {
        QueryWrapper<CustomerActionNote> qw = new QueryWrapper<>();
        qw.like(param.getName() != null, "t1.customerName", param.getName());
        qw.like(param.getTel() != null, "t1.tel", param.getTel());
        qw.like(param.getBrand() != null, "t1.brandList", param.getBrand());
        qw.like(param.getProviderName() != null, "t2.name", param.getProviderName());
        qw.inSql(param.getDepartmentId() != null,"t2.openid","SELECT openid from qywxDepartmentUserLink WHERE qywxDepartmentUserLink.departmentId = "+param.getDepartmentId());
        IPage<Map> iPage = customerActionNoteService.pageList(MybatisPlusUtil.SetNumPage(param.getPageNum(), param.getPageSize()), null, qw);
        List<Map> records = iPage.getRecords();
        for (Map m : records) {
            if (m.get("brandList") != null) {
                m.replace("brandList", new CustomUtil().stringToList(m.get("brandList").toString()));
            }
        }
        return Result.success(iPage);
    }

    @Operation(summary = "导出用户线索excel", description = "用户线索")
    @PostMapping("/cnExcel")
//    @AdminTokenValid
    public void cnExcel(HttpServletResponse response, @RequestBody CNListParam param) throws IOException {
        //这是表头及格式
        String[][] array = {
            {"姓名", "手机号", "地区", "关注车型", "备注", "关联任务", "提交人", "部门"},
            {"customerName", "tel", "area", "brandList", "remarks", "taskTitle", "provideName", "departmentName"},
            {"String", "String", "String", "String", "String", "String", "String", "String", "String", "String"}
        };
        QueryWrapper<CustomerActionNote> qw = new QueryWrapper<>();
        qw.like(param.getName() != null, "t1.customerName", param.getName());
        qw.like(param.getTel() != null, "t1.tel", param.getTel());
        qw.like(param.getBrand() != null, "t1.brandList", param.getBrand());
        qw.like(param.getProviderName() != null, "t2.name", param.getProviderName());
        qw.eq(param.getDepartmentId() != null, "t4.departmentId", param.getDepartmentId());
        IPage<Map> iPage = customerActionNoteService.pageList(MybatisPlusUtil.SetNumPage(1, 10000), null, qw);
        List<Map> records = iPage.getRecords();
        Workbook wb = ExcelUtil.writeToExcelByList(array, records);
        OutputStream output = response.getOutputStream();
        String fileName = "用户线索.xlsx";
        try {
            fileName = URLEncoder.encode(fileName, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        response.setHeader("Content-disposition", "attachment;filename=" + fileName + ";" + "filename*=utf-8''" + fileName);
        wb.write(output);
        output.close();
    }

}
