package com.zh.project0512.controller.manage;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.zh.project0512.annotation.AdminCheckAuthorityAnnotation;
import com.zh.project0512.annotation.AdminTokenValid;
import com.zh.project0512.model.TeachingRefe;
import com.zh.project0512.service.ITeachingRefeService;
import com.zh.project0512.utils.FileTypeUtil;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-06-14
 */
@RestController
@RequestMapping("/manage/teachingRefe")
@Tag(name = "通用教程")
public class TeachingRefeController {
    @Autowired
    private ITeachingRefeService teachingRefeService;

    @Data
    static class AddTCRParam {
        @NotNull(message = "title不能为空")
        @Schema(title = "标签名称")
        private String title;
        @NotNull(message = "教程文件地址不能为空")
        @Schema(title = "教程文件地址")
        private String fileUrl;
        @Schema(title = "时长", description = "视频文件请上传时长")
        private Integer duration;
    }

    @Operation(summary = "创建")
    @PostMapping("/add")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "115")
    public Result add(@Validated @RequestBody AddTCRParam param) {
        String fileUrl = param.getFileUrl();
        if (fileUrl.lastIndexOf(".") == -1) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "文件错误");
        }
        teachingRefeService.addTeachingRefe(new TeachingRefe().setFileUrl(fileUrl).setType(FileTypeUtil.getcontentTypeNum(fileUrl.substring(fileUrl.lastIndexOf("."))))
            .setTitle(param.getTitle()).setCreatAt(LocalDateTime.now()).setDuration(param.getDuration()));
        return Result.success("添加完成");
    }

    @Data
    static class DelTCRParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "教程id")
        private Integer id;
    }

    @Operation(summary = "删除(软删除)")
    @PostMapping("/del")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "117")
    public Result del(@Validated @RequestBody DelTCRParam param) {
        return MybatisPlusUtil.sqlResult(teachingRefeService.update(new UpdateWrapper<TeachingRefe>().eq("id", param.getId()).set("sortWeight",0).set("isDeleted", 1)), "删除");
    }

    @Data
    static class UpdTCRParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "教程id")
        private Integer id;
        @Schema(title = "教程名称")
        private String title;
        @Schema(title = "教程文件地址")
        private String fileUrl;
    }

    @Operation(summary = "更新教程")
    @PostMapping("/upd")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "116")
    public Result upd(@Validated @RequestBody UpdTCRParam param) {
        UpdateWrapper<TeachingRefe> up = new UpdateWrapper<>();
        up.eq("id", param.getId())
            .set(param.getTitle() != null, "title", param.getTitle())
            .set("updateAt", LocalDateTime.now());
        String fileUrl = param.getFileUrl();
        if (param.getFileUrl() != null) {
            up.set("fileUrl", param.getFileUrl());
            up.set("type", FileTypeUtil.getcontentTypeNum(fileUrl.substring(fileUrl.lastIndexOf("."))));
        }
        return MybatisPlusUtil.sqlResult(teachingRefeService.update(up), "修改");
    }

    @Data
    static class listTCRParam {
        private Integer pageNum;
        private Integer pageSize;
        @Schema(title = "教程名称")
        private String title;
    }

    @Operation(summary = "列表")
    @PostMapping("/list")
    @AdminTokenValid
    public Result list(@Validated @RequestBody listTCRParam param) {
        QueryWrapper<TeachingRefe> qw = new QueryWrapper<>();
        qw.eq("isDeleted",0).orderByAsc("sortWeight").orderByDesc("updateAt", "creatAt").like(param.getTitle() != null, "title", param.getTitle());
        return Result.success(teachingRefeService.pageMaps(MybatisPlusUtil.SetNumPage(param.getPageNum(), param.getPageSize()), qw));
    }

    @Data
    static class UpdTRSParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "教程id")
        private Integer id;
        @NotNull(message = "排序权重不能为空")
        @Min(value = 1, message = "排序权重最小值为1")
        @Schema(title = "排序权重:数字越小排序靠前")
        private Integer sortWeight;
    }

    @Operation(summary = "修改排序")
    @PostMapping("/updSort")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "136")
    public Result updSort(@Validated @RequestBody UpdTRSParam param) {
        TeachingRefe teachingRefe = teachingRefeService.getById(param.getId());
        if(teachingRefe.getIsDeleted() == 1){
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION,"该标签已被删除");
        }
        int origin = teachingRefe.getSortWeight();
        int current = param.getSortWeight();
        if (origin == current) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "权重无变化");
        }
        teachingRefeService.updSort(origin, current, origin > current);
        return Result.success("修改完成");
    }

}
