package com.zh.project0512.controller.manage;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zh.project0512.annotation.AdminCheckAuthorityAnnotation;
import com.zh.project0512.annotation.AdminTokenValid;
import com.zh.project0512.model.*;
import com.zh.project0512.service.*;
import com.zh.project0512.utils.ExcelUtil;
import com.zh.project0512.utils.HttpUtil;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-05-18
 */
@Tag(name = "成员管理")
@RestController
@RequestMapping("/manage/user")
public class UserController {


    @Autowired
    private HttpUtil httpUtil;

    @Autowired
    private IUserService userService;

    @Autowired
    private IQywxDepartmentService qywxDepartmentService;

    @Autowired
    private IQywxDepartmentGroupService qywxDepartmentGroupService;

    @Autowired
    private IQywxDepartmentGroupLinkService qywxDepartmentGroupLinkService;

    @Autowired
    private IUserPointsRecordsService userPointsRecordsService;

    @Data
    static class UParam {
        private Integer pageNum;
        private Integer pageSize;
        @Schema(title = "人员名称")
        private String name;
        @Schema(title = "部门名称")
        private String department;
    }

    @Operation(summary = "成员列表")
    @PostMapping("/list")
    @AdminTokenValid
    public Result list(@Validated @RequestBody UParam param) {
        QueryWrapper<User> qw = new QueryWrapper<>();
        qw
//            .and(param.getDepartment() != null, q -> q.like("t2.name", param.getDepartment())
//                .or(o -> o.like("t3.name", param.getDepartment())))
               .inSql(param.getDepartment() != null,"t1.openid",
                   "SELECT t1.openid from qywxDepartmentUserLink as t1 INNER JOIN qywxDepartment as t2 on t1.departmentId = t2.departmentId and t2.name like "+"'%"+param.getDepartment()+"%'")
            .like(param.getName() != null, "t1.name", param.getName());
        return Result.success(userService.userDpList(MybatisPlusUtil.SetNumPage(param.getPageNum(), param.getPageSize()), qw));
    }

    @Operation(summary = "导出excel", description = "成员导出")
    @PostMapping("/excel")
    @AdminTokenValid
    public void excel(HttpServletResponse response, @Validated @RequestBody UParam param) throws IOException {
        //这是表头及格式
        String[][] array = {
            {"员工", "部门", "上级部门","部门列表", "手机号", "积分", "历史积分", "创建时间", "上次登录"},
            {"name", "main_departmentName", "sub_departmentName", "departmentList","tel", "points", "historyPoints", "creatAt", "lastLoginAt"},
            {"String", "String", "String", "String","String",  "int", "int", "LocalDateTime", "LocalDateTime"}
        };
        QueryWrapper<User> qw = new QueryWrapper<>();
        qw
            .and(param.getDepartment() != null, q -> q.like("t2.name", param.getDepartment())
                .or(o -> o.like("t3.name", param.getDepartment())))
            .like(param.getName() != null, "t1.name", param.getName());
        IPage<Map> iPage = userService.userDpList(MybatisPlusUtil.SetNumPage(1, 10000), qw);
        List<Map> records = iPage.getRecords();
        for (Map map:records){
            List<HashMap> departmentList = (List) map.get("departmentList");
            StringBuffer s = new StringBuffer();
            departmentList.forEach(v->s.append(v.get("departmentName")+"，"));
            String s1 = s.toString();
            map.replace("departmentList", StringUtils.isBlank(s)?"":(s1.substring(0, s1.length()-1)));
        }
        Workbook wb = ExcelUtil.writeToExcelByList(array,records );
        OutputStream output = response.getOutputStream();
        String fileName = "成员导出.xlsx";
        try {
            fileName = URLEncoder.encode(fileName, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        response.setHeader("Content-disposition", "attachment;filename=" + fileName + ";" + "filename*=utf-8''" + fileName);
        wb.write(output);
        output.close();
    }

    @Data
    static class PTParam {
        @NotNull(message = "用户id不能为空")
        @Min(value = 1, message = "用户id最小值为1")
        @Schema(title = "用户账号id")
        private Integer userId;
        @NotNull(message = "结算方式不能为空")
        @Min(value = 1, message = "1.增加，2.扣除")
        @Max(value = 2, message = "1.增加，2.扣除")
        @Schema(title = "结算方式", description = "1.增加，2.扣除")
        private Integer settlementMethod;
        @NotNull(message = "积分不能为空")
        @Schema(title = "变动积分")
        private Integer points;
        @Schema(title = "备注")
        private String remarks;
    }

    @Data
    static class DetUParam {
        private Integer pageNum;
        private  Integer pageSize;
        @NotNull(message = "userId不能为空")
        @Min(value = 1, message = "userId最小值为1")
        @Schema(title = "用户id")
        private Integer userId;
        @Schema(title = "类型",description = "1.任务奖励，2.素材分享奖励，3.线下结算扣除 4.后台更改")
        private Integer type;
        @Schema(title = "结算方式",description = "1.增加，2.扣除")
        private Integer settlementMethod;
    }
    @Operation(summary = "用户积分记录")
    @PostMapping("/pointsRecords")
    @Transactional
    @AdminTokenValid
    public Result pointsRecords(@Validated @RequestBody DetUParam param) {
        QueryWrapper<UserPointsRecords> qw = new QueryWrapper<>();
        qw.eq("userId",param.getUserId());
        qw.eq(param.getType()!=null,"type",param.getType());
        qw.eq(param.getSettlementMethod()!=null,"type",param.getSettlementMethod());
        qw.select("*,(case when type = 0 then '无' when type = 1 then '任务奖励'" +
            " when type =2 then '素材分享奖励' when type = 3 then '线下结算扣除' when type = 4 then '后台更改' else '其他' end) as typeDSC," +
            "(case when settlementMethod = 1 then '增加' when settlementMethod = 2 then '扣除' else '无' end) as settlementMethodDSC ");
       return Result.success(userPointsRecordsService.page(MybatisPlusUtil.SetNumPage(param.getPageNum(), param.getPageSize()),qw));
    }

    @Operation(summary = "修改积分")
    @PostMapping("/pointsTran")
    @Transactional
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "138")
    public Result pointsTran(@Validated @RequestBody PTParam param) {
        UserPointsRecords userPointsRecords = new UserPointsRecords();
        Integer userId = param.getUserId();
        Integer points = param.getPoints();
        Integer settlementMethod = param.getSettlementMethod();
        User user = userService.getById(userId);
        if (null == user) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "未找到用户");
        }
        Integer pointsCur = user.getPoints();
        UpdateWrapper<User> up = new UpdateWrapper<>();
        up.eq("id", userId);
        if (settlementMethod == 1) {
            up.set("points", pointsCur + points).set("historyPoints", user.getHistoryPoints() + points);
        } else if (settlementMethod == 2) {
            if (pointsCur < points) {
                return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "积分不足");
            }
            up.set("points", pointsCur - points);
        }
        userService.update(up);
        userPointsRecordsService.save(userPointsRecords.setPoints(param.getPoints()).setUserId(param.getUserId()).setSettlementMethod(param.getSettlementMethod()).setRemarks(param.getRemarks()).setCreateDate(LocalDateTime.now()));
        return Result.success(null, "操作完成");
    }

    @Operation(summary = "部门列表")
    @PostMapping("/department")
    @AdminTokenValid
    public Result department(@RequestBody JSONObject obj) {
        return Result.success(qywxDepartmentService.list(MybatisPlusUtil.SetPage(obj)));
    }

    @Operation(summary = "同步企业微信部门信息", description = "当企业微信架构改动更新时，需手动触发同步信息")
    @PostMapping("/departmentRefresh")
    @AdminTokenValid
    public Result departmentRefresh() {
        String access_token = httpUtil.qywxGetToken();
        JSONObject res = httpUtil.qywxDepartmentList(access_token);
        List department = res.getJSONArray("department");
        qywxDepartmentService.remove(new QueryWrapper<>());
        qywxDepartmentService.updateAll(department);
        return Result.success(res.getJSONArray("department"));
    }

    @Data
    static class AddDGParam {
        @NotNull(message = "name不能为空")
        @Schema(title = "部门级别名称")
        private String name;
    }

    @Operation(summary = "新增部门级别")
    @PostMapping("/addDepartmentGroup")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "118")
    public Result addDepartmentGroup(@Validated @RequestBody AddDGParam param) {
        return Result.success(qywxDepartmentGroupService.save(new QywxDepartmentGroup().setName(param.getName()).setCreatAt(LocalDateTime.now())));
    }


    @Operation(summary = "部门级别列表")
    @PostMapping("/departmentGroup")
    @AdminTokenValid
    public Result departmentGroup(@RequestBody JSONObject obj) {
        return Result.success(qywxDepartmentGroupService.page(MybatisPlusUtil.SetPage(obj)));
    }

    @Data
    static class DelDGParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "部门级别id")
        private Integer id;
    }

    @Operation(summary = "部门级别删除")
    @PostMapping("/delDepartmentGroup")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "119")
    public Result delDepartmentGroup(@Validated @RequestBody DelDGParam param) {
        return MybatisPlusUtil.sqlResult(qywxDepartmentGroupService.removeById(param.getId()), "删除");
    }

    @Data
    static class UpdDGParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "部门级别id")
        private Integer id;
        @NotNull(message = "name不能为空")
        @Schema(title = "部门级别名称")
        private String name;
    }

    @Operation(summary = "部门级别重命名")
    @PostMapping("/renameDepartmentGroup")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "139")
    public Result departmentGroup(@Validated @RequestBody UpdDGParam param) {
        UpdateWrapper<QywxDepartmentGroup> up = new UpdateWrapper<>();
        up.eq("id", param.getId()).set("name", param.getName()).set("updateAt", LocalDateTime.now());
        return MybatisPlusUtil.sqlResult(qywxDepartmentGroupService.update(up), "修改");
    }

    @Data
    static class dgAddParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "部门级别id")
        private Integer id;
        @NotNull(message = "部门id列表不能为空")
        @Schema(title = "部门id列表", description = "格式为[1,...,2]")
        private List<Integer> departmentIdList;
    }

    @Operation(summary = "部门级别添加部门", description = "批量添加部门id")
    @PostMapping("/departmentGroupAdd")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "139")
    public Result departmentGroupAdd(@Validated @RequestBody dgAddParam param) {
        qywxDepartmentGroupLinkService.groupAddDepartment(param.getId(), param.getDepartmentIdList(), LocalDateTime.now());
        return Result.success("添加完成");
    }

    @Operation(summary = "部门级别详情")
    @PostMapping("/departmentGroupDet")
    @AdminTokenValid
    public Result departmentGroupDet(@Validated @RequestBody DelDGParam param) {
        return Result.success(qywxDepartmentGroupLinkService.departmentList(param.getId()));
    }

    @Operation(summary = "部门级别详情2")
    @PostMapping("/departmentGroupDet2")
    @AdminTokenValid
    public Result departmentGroupDet2(@Validated @RequestBody DelDGParam param) {
        return Result.success(qywxDepartmentGroupLinkService.departmentList2(param.getId()));
    }

    @Operation(summary = "部门级别删除部门", description = "批量删除部门id")
    @PostMapping("/departmentGroupDel")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "139")
    public Result departmentGroupDel(@Validated @RequestBody dgAddParam param) {
        QueryWrapper<QywxDepartmentGroupLink> qw = new QueryWrapper<>();
        qw.eq("groupId", param.getId());
        qw.in("departmentId", param.getDepartmentIdList());
        qywxDepartmentGroupLinkService.remove(qw);
        return Result.success("删除完成");
    }

}

