package com.zh.project0512.controller.wxApp;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.zh.project0512.annotation.AdminTokenValid;
import com.zh.project0512.annotation.TokenValid;
import com.zh.project0512.controller.manage.StatisticsController;
import com.zh.project0512.mapper.UserEffectRecordsMapper;
import com.zh.project0512.model.UserEffectRecords;
import com.zh.project0512.model.dto.RankListDTO;
import com.zh.project0512.service.IRankService;
import com.zh.project0512.service.IVideoService;
import com.zh.project0512.utils.JwtUtil;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.TemporalAdjusters;

/**
 * 排行榜
 */
@Tag(name = "排行榜")
@RestController
@RequestMapping("/wxApp/rank")
public class RankUController {
    @Resource
    IRankService rankService;
    @Autowired
    private IVideoService videoService;
    @Autowired
    private UserEffectRecordsMapper userEffectRecordsMapper;
//    /**
//     * 查询排行榜列表
//     *
//     * @param rankListDTO 排行列表DTO
//     * @return 排行榜列表
//     */
//    @Operation(summary = "排行列表")
//    @PostMapping("/list")
//    public Result rankList(@Validated @RequestBody RankListDTO rankListDTO) {
//        return Result.success(rankService.rankList(rankListDTO), "请求成功");
//    }

    @Data
    static class RankParam {
        //        @Schema(title = "时间条件", description = "1今日;2本周;3本月;4全年")
//        private Integer dateCondition;
        @Schema(title = "月份", description = "格式为yyyy-MM-dd HH:mm:ss")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime dateTime;
        @Schema(title = "1经销商2员工", description = "默认1")
        private Integer type;
        private Integer pageNum;
        private Integer pageSize;
    }

    @Operation(summary = "排行列表")
    @PostMapping("/list")
    @AdminTokenValid
    public Result list(@Validated @RequestBody RankParam param) {
        Page pg = MybatisPlusUtil.SetNumPage(param.getPageNum(), param.getPageSize());
        QueryWrapper<UserEffectRecords> qw = new QueryWrapper<>();
        LocalDateTime now = param.getDateTime() == null ? LocalDateTime.now() : param.getDateTime();
        qw.eq("date_format(createAt, '%Y-%m')", now.format(DateTimeFormatter.ofPattern("yyyy-MM")));
        IPage page = (param.getType() != null && param.getType() == 1) ? userEffectRecordsMapper.rankByDepartment(pg, qw) : userEffectRecordsMapper.rankByUser(pg, qw);
        return Result.success(page.getRecords(), "请求成功");
    }

    @Operation(summary = "个人总记录")
    @PostMapping("/myEffect")
    @TokenValid
    public Result myEffect(@RequestHeader String token) {
        return Result.success(videoService.record(new JwtUtil().parseOpenid(token)), "请求成功");
    }
}
