package com.zh.project0512.controller.wxApp;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.zh.project0512.model.TopicActivity;
import com.zh.project0512.service.ITopicActivityService;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-05-26
 */
@Tag(name = "动态")
@RestController
@RequestMapping("/wxApp/topicActivity")
public class TopicActivityUController {
    @Autowired
    private ITopicActivityService topicActivityService;

    @Operation(summary = "动态列表")
    @PostMapping("/list")
    public Result list(@RequestBody(required = false) JSONObject obj) {
        QueryWrapper<TopicActivity> qw = new QueryWrapper<>();
        if(obj.getBoolean("showAtIndex") !=null && obj.getBoolean("showAtIndex") == true){
            qw.eq("showAtIndex",1);
            qw.orderByDesc("showAtIndex");
        }
        qw.orderByAsc("sortWeight");
        qw.orderByDesc("updateAt","creatAt");

        return Result.success(topicActivityService.page(MybatisPlusUtil.SetPage(obj),qw));
    }

    @Data
    static class DetTaskParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "动态id")
        private Integer id;
    }

    @Operation(summary = "详情")
    @PostMapping("/detail")
    public Result detail(@Validated @RequestBody DetTaskParam param) {
        int id =param.getId();
        TopicActivity topicActivity = topicActivityService.getById(id);
        return topicActivity != null ? Result.success(topicActivity) : Result.fail(HttpStatusEnum.NOT_FOUND);
    }
    @Operation(summary = "点击量增加")
    @PostMapping("/noteHits")
    public Result noteHits(@Validated @RequestBody DetTaskParam param) {
        UpdateWrapper wrapper = new UpdateWrapper();
        wrapper.eq("id",param.getId());
        wrapper.setSql("playNum = playNum + 1");
        return MybatisPlusUtil.sqlResult( topicActivityService.update(wrapper),"点击量记录");
    }
}
