package com.zh.project0512.controller.wxApp;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.zh.project0512.annotation.TokenValid;
import com.zh.project0512.model.UserAccount;
import com.zh.project0512.service.IUserAccountService;
import com.zh.project0512.service.IUserService;
import com.zh.project0512.utils.JwtUtil;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-06-08
 */
@Tag(name = "用户账号关联")
@RestController
@RequestMapping("/wxApp/userAccount")
public class UserAccountUController {

    @Autowired
    private IUserAccountService userAccountService;

    @Autowired
    private IUserService userService;

    @Data
    static class AddUAParam {
        @NotNull(message = "账号名称不能为空")
        private String name;
        @NotNull(message = "手机号不能为空")
//    @Pattern(regexp = "^[1]\\\\d{10}$",message = "请输入正确的手机号")
        private String tel;
        @NotNull(message = "平台信息不能为空")
        @Schema(title = "平台", description = "1抖音；2快手；3朋友圈；4视频号")
        @Min(value = 1, message = "选择范围为：1抖音；2快手；3朋友圈；4视频号")
        @Max(value = 4, message = "选择范围为：1抖音；2快手；3朋友圈；4视频号")
        private Integer platform;
        @Schema(title = "验证图片")
        @NotNull(message = "验证图片不能为空")
        private String validImg;
        @Schema(title = "平台账号")
        private String account;
    }

    @Operation(summary = "添加关联账号")
    @PostMapping("/add")
    @TokenValid
    public Result add(@Validated @RequestBody AddUAParam param, @RequestHeader String token) {
        Integer userId = userService.selByOpenid(new JwtUtil().parseOpenid(token)).getId();
        if (userId == null) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "未找到该用户");
        }
        UserAccount ua = new UserAccount();
        ua.setUserId(userId).setPlatform(param.getPlatform()).setName(param.getName()).setTel(param.getTel())
            .setAccount(param.getAccount()).setValidImg(param.getValidImg()).setStatus(0).setCreatAt(LocalDateTime.now());
        QueryWrapper<UserAccount> qw = new QueryWrapper<>();
        qw.eq("userId", userId).eq("platform", param.getPlatform());
        userAccountService.saveOrUpdate(ua, qw);
        return Result.success("添加完成，请等待审核");
    }

    @Data
    static class editUAParam {
        private String name;
        //    @Pattern(regexp = "^[1]\\\\d{10}$",message = "请输入正确的手机号")
        private String tel;
        @NotNull(message = "平台id不能为空")
        @Schema(title = "平台", description = "1抖音；2快手；3朋友圈；4视频号")
        @Min(value = 1, message = "选择范围为：1抖音；2快手；3朋友圈；4视频号")
        @Max(value = 4, message = "选择范围为：1抖音；2快手；3朋友圈；4视频号")
        private Integer platform;
        @Schema(title = "验证图片")
        private String validImg;
        @Schema(title = "平台账号")
        private String account;
    }

    @Operation(summary = "编辑关联账号")
    @PostMapping("/edit")
    @TokenValid
    public Result edit(@Validated @RequestBody editUAParam param, @RequestHeader String token) {
        Integer userId = userService.selByOpenid(new JwtUtil().parseOpenid(token)).getId();
        if (userId == null) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "未找到该用户");
        }
        UserAccount ua = new UserAccount();
        String name = param.getName();
        String tel = param.getTel();
        String validImg = param.getValidImg();
        ua.setStatus(0).setUpdateAt(LocalDateTime.now());
        UpdateWrapper<UserAccount> qw = new UpdateWrapper<>();
        qw.eq("userId", userId).eq("platform", param.getPlatform());
        qw.set(name != null, "name", name)
            .set(tel != null, "tel", tel)
            .set(param.getAccount()!=null,"account",param.getAccount())
            .set(validImg != null, "validImg", validImg);
        userAccountService.saveOrUpdate(ua, qw);
        return Result.success("编辑完成，请等待审核");
    }

    @Operation(summary = "关联账号列表")
    @PostMapping("/list")
    @TokenValid
    public Result list(@RequestHeader String token) {
        Integer userId = userService.selByOpenid(new JwtUtil().parseOpenid(token)).getId();
        if (userId == null) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "未找到该用户");
        }
        QueryWrapper<UserAccount> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId", userId);
        queryWrapper.orderByDesc("updateAt", "creatAt");
        return Result.success(userAccountService.list(queryWrapper));
    }

    @Data
    static class detailUAParam {
        @NotNull(message = "平台id不能为空")
        @Schema(title = "平台", description = "1抖音；2快手；3朋友圈；4视频号")
        @Min(value = 1, message = "选择范围为：1抖音；2快手；3朋友圈；4视频号")
        @Max(value = 4, message = "选择范围为：1抖音；2快手；3朋友圈；4视频号")
        private Integer platform;
    }

    @Operation(summary = "关联账号详情")
    @PostMapping("/detail")
    @TokenValid
    public Result detail(@Validated @RequestBody detailUAParam param, @RequestHeader String token) {
        Integer userId = userService.selByOpenid(new JwtUtil().parseOpenid(token)).getId();
        if (userId == null) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "未找到该用户");
        }
        QueryWrapper<UserAccount> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId", userId).eq("platform",param.getPlatform());
        queryWrapper.orderByDesc("updateAt", "creatAt");
        return Result.success(userAccountService.getOne(queryWrapper));
    }
}
