package com.zh.project0512.controller.wxApp;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.zh.project0512.annotation.TokenValid;
import com.zh.project0512.model.Task;
import com.zh.project0512.model.UserTask;
import com.zh.project0512.service.ITaskService;
import com.zh.project0512.service.IUserService;
import com.zh.project0512.service.IUserTaskService;
import com.zh.project0512.utils.JwtUtil;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-05-27
 */
@Tag(name = "用户任务")
@RestController
@RequestMapping("/wxApp/userTask")
public class UserTaskUController {
    @Autowired
    private IUserTaskService userTaskService;
    @Autowired
    private ITaskService taskService;
    @Autowired
    private IUserService userService;

    @Data
    static class AddUTParam {
        @Schema(title = "任务id")
        @NotNull(message = "taskId不能为空")
        @Min(value = 1, message = "taskId最小值为1")
        private Integer taskId;
    }

    @Operation(summary = "用户接受任务")
    @TokenValid
    @PostMapping("/add")
    public Result add(@Validated @RequestBody AddUTParam param, @RequestHeader("token") @Parameter(name = "登录token") String token) {
        Integer userId = userService.selByOpenid(new JwtUtil().parseOpenid(token)).getId();
        Integer taskId = param.getTaskId();
        if(userId == null){
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION,"用户不存在");
        }
        QueryWrapper<UserTask> queryWrapper = new QueryWrapper<>();
        Task task = taskService.getById(taskId);
        if(null == task){
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION,"任务不存在");
        }
        if(task.getStatus()!=1){
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION,"任务不在接受时间内");
        }
        if (null != userTaskService.getMap(new QueryWrapper<UserTask>().eq("userId", userId).eq("taskId", taskId))) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION,"已接受该任务");
        }
        userTaskService.save(new UserTask().setUserId(userId).setTaskId(taskId));
        return Result.success("添加完成");
    }
}
