package com.zh.project0512.serviceImpl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.zh.project0512.mapper.*;
import com.zh.project0512.model.*;
import com.zh.project0512.model.dto.OperationRoleJurisdictionDTO;
import com.zh.project0512.model.dto.AdminJurisdictionFBIDTO;
import com.zh.project0512.model.vo.AdminJurisdictionFBRIOperationVo;
import com.zh.project0512.model.vo.AdminJurisdictionFBRIVo;
import com.zh.project0512.service.IAdminJurisdictionService;
import com.zh.project0512.utils.BizException;
import com.zh.project0512.utils.PropertyUtils;
import com.zh.project0512.utils.result.HttpStatusEnum;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@Service
public class AdminJurisdictionServiceImpl implements IAdminJurisdictionService {
    @Resource
    AdminRoleJurisdictionMapper adminRoleJurisdictionMapper;
    @Resource
    AdminMenusMapper adminMenusMapper;
    @Resource
    AdminOperationMapper adminOperationMapper;
    @Resource
    AdminJurisdictionMapper adminJurisdictionMapper;
    @Resource
    AdminRoleMapper adminRoleMapper;

    @Override
    @Transactional
    public boolean operationRoleJurisdiction(OperationRoleJurisdictionDTO operationRoleJurisdictionDTO) {
        AdminRole adminRole = adminRoleMapper.selectById(operationRoleJurisdictionDTO.getRoleId());
        if (adminRole == null){
            throw new BizException(HttpStatusEnum.CUSTOM_EXCEPTION,"记录不存在");
        }

        //先删除角色-权限关联表
        QueryWrapper<AdminRoleJurisdiction> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("roleId",operationRoleJurisdictionDTO.getRoleId());
        adminRoleJurisdictionMapper.delete(queryWrapper);

        //后添加角色-权限关联表
        int[] jurisdictionIds = operationRoleJurisdictionDTO.getJurisdictionIds();
        if (jurisdictionIds.length > 0){
            for (int jurisdictionId : jurisdictionIds) {
                AdminRoleJurisdiction adminRoleJurisdiction = new AdminRoleJurisdiction();
                adminRoleJurisdiction.setRoleId(operationRoleJurisdictionDTO.getRoleId());
                adminRoleJurisdiction.setJurisdictionId(jurisdictionId);
                adminRoleJurisdictionMapper.insert(adminRoleJurisdiction);
            }
        }

        return true;
    }

    @Override
    public List<AdminJurisdictionFBRIVo> findByRoleId(AdminJurisdictionFBIDTO adminJurisdictionFBIDTO) {
        AdminRole adminRole = adminRoleMapper.selectById(adminJurisdictionFBIDTO.getRoleId());
        if (adminRole == null){
            throw new BizException(HttpStatusEnum.CUSTOM_EXCEPTION,"记录不存在");
        }

        ArrayList<AdminJurisdictionFBRIVo> adminJurisdictionFBRIVos = new ArrayList<>();

        //查询所有菜单
        QueryWrapper<AdminMenus> queryWrapper = new QueryWrapper<>();
        List<AdminMenus> adminMenus = adminMenusMapper.selectList(queryWrapper);
        if (adminMenus != null && adminMenus.size() >0){
            for (AdminMenus userMenu : adminMenus) {
                AdminJurisdictionFBRIVo adminJurisdictionFBRIVo = new AdminJurisdictionFBRIVo();
                PropertyUtils.copyProperties(userMenu, adminJurisdictionFBRIVo);
                adminJurisdictionFBRIVo.setCheck(false);//默认为未选中

                //查询菜单下所有操作信息
                ArrayList<AdminJurisdictionFBRIOperationVo> operationVoList = new ArrayList<>();
                QueryWrapper<AdminJurisdiction> queryWrapper4 = new QueryWrapper<>();
                queryWrapper4.eq("menusId",userMenu.getId());
                List<AdminJurisdiction> adminJurisdictions = adminJurisdictionMapper.selectList(queryWrapper4);
                if (adminJurisdictions != null && adminJurisdictions.size()>0){
                    for (AdminJurisdiction adminJurisdiction : adminJurisdictions) {
//                        AdminOperation adminOperation = adminOperationMapper.selectById(adminJurisdiction.getOperationId());
//                        if (adminOperation != null){
                            AdminJurisdictionFBRIOperationVo operationVo = new AdminJurisdictionFBRIOperationVo();
//                            PropertyUtils.copyProperties(adminOperation,operationVo);
                            operationVo.setId(adminJurisdiction.getId());
                        operationVo.setName(adminJurisdiction.getRemake());
                            operationVo.setCheck(false);//默认为未选中
                            //查询该操作是否选中,如果选中则菜单也被选中
                            QueryWrapper<AdminRoleJurisdiction> queryWrapper5 = new QueryWrapper<>();
                            queryWrapper5.eq("jurisdictionId", adminJurisdiction.getId());
                            queryWrapper5.eq("roleId", adminJurisdictionFBIDTO.getRoleId());
                            queryWrapper5.last("limit 1");
                            AdminRoleJurisdiction adminRoleJurisdiction = adminRoleJurisdictionMapper.selectOne(queryWrapper5);
                            if (adminRoleJurisdiction != null){
                                operationVo.setCheck(true);
                                adminJurisdictionFBRIVo.setCheck(true);
                            }

                            operationVoList.add(operationVo);
//                        }
                    }
                }

                adminJurisdictionFBRIVo.setOperationVoList(operationVoList);
                adminJurisdictionFBRIVos.add(adminJurisdictionFBRIVo);
            }
        }

        return adminJurisdictionFBRIVos;
    }
}
